@@ -266,6 +266,12 @@ read_item_common (MapiItem *item, uint32_t ulPropTag, gconstpointer prop_data)
 	case PR_IN_REPLY_TO_ID:
 		item->header.in_reply_to = g_strdup (prop_data);
 		break;
+	case PR_TRANSPORT_MESSAGE_HEADERS:
+		sv (item->header.transport_headers, utf8tolinux (prop_data));
+		break;
+	case PR_TRANSPORT_MESSAGE_HEADERS_UNICODE:
+		sv (item->header.transport_headers, g_strdup (prop_data));
+		break;
 	default:
 		found = FALSE;
 		break;
@@ -685,7 +691,6 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	guint32 index, count, options = 0;
 	GSList *server_uid_list = NULL;
 	const gchar *uid = NULL;
-	GSList *tmp_list_item = NULL;
 
 	/* Currently we don't have simple wrapper over getprops.*/
 	const guint32 prop_list[] = { PR_LAST_MODIFICATION_TIME };
@@ -705,7 +710,7 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 					       prop_list, G_N_ELEMENTS (prop_list),
 					       NULL, NULL,
 					       deleted_items_sync_cb, &server_uid_list,
-					       options);
+					       options | MAPI_OPTIONS_DONT_OPEN_MESSAGE);
 
 	camel_operation_end (NULL);
 
@@ -722,6 +727,8 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 
 	/* Iterate over cache and check if the UID is in server*/
 	for (index = 0; index < count; index++) {
+		GSList *tmp_list_item = NULL;
+
 		/*FIXME :Any other list available ???*/
 		info = camel_folder_summary_index (m->folder->summary, index);
 		if (!info) continue; /*This is bad. *Should* not happen*/
@@ -735,7 +742,7 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 		}
 
 		/* If it is not in server list, clean our cache */
-		if (!tmp_list_item->data && uid) {
+		if ((!tmp_list_item || !tmp_list_item->data) && uid) {
 			CAMEL_MAPI_FOLDER_REC_LOCK (m->folder, cache_lock);
 			camel_folder_summary_remove_uid (m->folder->summary, uid);
 			camel_data_cache_remove (mapi_folder->cache, "cache", uid, NULL);
@@ -953,7 +960,9 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 		PR_IN_REPLY_TO_ID,
 		PR_DISPLAY_TO,
 		PR_DISPLAY_CC,
-		PR_DISPLAY_BCC
+		PR_DISPLAY_BCC,
+		PR_TRANSPORT_MESSAGE_HEADERS,
+		PR_TRANSPORT_MESSAGE_HEADERS_UNICODE
 	};
 
 	if (((CamelOfflineStore *) mapi_store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
@@ -1081,6 +1090,8 @@ static const uint32_t camel_GetPropsList[] = {
 	PR_MID, 
 	PR_INTERNET_CPID,
 
+	PR_TRANSPORT_MESSAGE_HEADERS,
+	PR_TRANSPORT_MESSAGE_HEADERS_UNICODE,
 	PR_MESSAGE_CLASS, 
 	PR_MESSAGE_CLASS_UNICODE, 
 	PR_MESSAGE_SIZE, 
@@ -1368,7 +1379,7 @@ mapi_msg_set_recipient_list (CamelMimeMessage *msg, MapiItem *item)
 	}
 	
 	/*Add to message*/
-	camel_mime_message_set_recipients(msg, "To", to_addr);
+	/*Note : To field is added from PR_TRANSPORT_MESSAGE_HEADERS*/
 	camel_mime_message_set_recipients(msg, "Cc", cc_addr);
 	camel_mime_message_set_recipients(msg, "Bcc", bcc_addr);
 
@@ -1384,10 +1395,39 @@ mapi_populate_details_from_item (CamelFolder *folder, CamelMimeMessage *msg, Map
 	time_t recieved_time;
 	CamelInternetAddress *addr = NULL;
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE(folder->parent_store);
-
 	int offset = 0;
 	time_t actual_time;
 
+	/* Setting headers from PR_TRANSPORT_MESSAGE_HEADERS */
+	if (item->header.transport_headers) {
+		CamelMimePart *part = camel_mime_part_new ();
+		CamelStream *stream;
+		CamelMimeParser *parser;
+
+		stream = camel_stream_mem_new_with_buffer (item->header.transport_headers, strlen (item->header.transport_headers));
+		parser = camel_mime_parser_new ();
+		camel_mime_parser_init_with_stream (parser, stream);
+		camel_mime_parser_scan_from (parser, FALSE);
+		camel_object_unref (stream);
+
+		if (camel_mime_part_construct_from_parser (part, parser) != -1) {
+			struct _camel_header_raw *h;
+
+			for (h = part->headers; h; h = h->next) {
+				const gchar *value = h->value;
+
+				while (value && camel_mime_is_lwsp (*value))
+					value++;
+
+				camel_medium_add_header (CAMEL_MEDIUM (msg), h->name, value);
+			}
+		}
+
+		camel_object_unref (parser);
+		camel_object_unref (part);
+	}
+
+	/* Overwrite headers if we have specific properties available*/
 	temp_str = item->header.subject;
 	if(temp_str) 
 		camel_mime_message_set_subject (msg, temp_str);
@@ -1395,7 +1435,7 @@ mapi_populate_details_from_item (CamelFolder *folder, CamelMimeMessage *msg, Map
 	recieved_time = item->header.recieved_time;
 
 	actual_time = camel_header_decode_date (ctime(&recieved_time), &offset);
-	camel_mime_message_set_date (msg, actual_time, offset);
+	/* camel_mime_message_set_date (msg, actual_time, offset); */
 
 	if (item->header.from) {
 		if ((item->header.from_type != NULL) && !g_utf8_collate (item->header.from_type, "EX")) {
@@ -1496,7 +1536,7 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 
 	/* Threading */
 	if (item->header.message_id)
-		camel_medium_add_header (CAMEL_MEDIUM (msg), "Message-Id", item->header.message_id);
+		camel_medium_add_header (CAMEL_MEDIUM (msg), "Message-ID", item->header.message_id);
 
 	if (item->header.references)
 		camel_medium_add_header (CAMEL_MEDIUM (msg), "References", item->header.references);
@@ -1949,7 +1989,7 @@ mapi_append_message (CamelFolder *folder, CamelMimeMessage *message,
 
 	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, 
-				      _("Offline. '%s'"));
+				 _("Offline. '%s'"));
 		return;
 	}
 
