@@ -70,6 +70,7 @@ mapi_profile_load (const char *profname, const char *password)
 	enum MAPISTATUS	retval = MAPI_E_SUCCESS;
 	struct mapi_session *session = NULL;
 	gchar *profpath = NULL;
+	gchar *default_profile_name = NULL;
 	const char *profile = NULL;
 	guint32 debug_log_level = 0;
 
@@ -100,13 +101,13 @@ mapi_profile_load (const char *profname, const char *password)
 	if (profname)
 		profile = profname;
 	else {
-		retval = GetDefaultProfile (&profile);
+		retval = GetDefaultProfile (&default_profile_name);
 		if (retval != MAPI_E_SUCCESS) {
 			mapi_errstr("GetDefaultProfile", GetLastError());
 			goto cleanup;
 		}
+		profile = default_profile_name;
 	}
-
 	g_print("\nLoading profile %s ", profile);
 
 	retval = MapiLogonEx(&session, profile, password);
@@ -719,16 +720,17 @@ mapidump_PAB_gal_entry (struct SRow *aRow)
 	const char	*name;
 	const char	*email;
 	const char	*account;
+	ExchangeMAPIGALEntry *gal_entry;
 
 	addrtype = (const char *)find_SPropValue_data(aRow, PR_ADDRTYPE_UNICODE);
 	name = (const char *)find_SPropValue_data(aRow, PR_DISPLAY_NAME_UNICODE);
-	email = (const char *)find_SPropValue_data(aRow, PR_EMAIL_ADDRESS_UNICODE);
+	email = (const char *)find_SPropValue_data(aRow, PR_SMTP_ADDRESS_UNICODE);
 	account = (const char *)find_SPropValue_data(aRow, PR_ACCOUNT_UNICODE);
 
 	printf("[%s] %s:\n\tName: %-25s\n\tEmail: %-25s\n", 
 	       addrtype, account, name, email);
 
-	ExchangeMAPIGALEntry *gal_entry = g_new0 (ExchangeMAPIGALEntry, 1);
+	gal_entry = g_new0 (ExchangeMAPIGALEntry, 1);
 	gal_entry->name = g_strdup (name);
 	gal_entry->email = g_strdup (email);
 
@@ -748,11 +750,13 @@ exchange_mapi_util_get_gal (GPtrArray *contacts_array)
 	
 	mem_ctx = talloc_init ("ExchangeMAPI_GetGAL");
 
+	LOCK ();
+
 	SPropTagArray = set_SPropTagArray(mem_ctx, 0xc,
 					  PR_INSTANCE_KEY,
 					  PR_ENTRYID,
 					  PR_DISPLAY_NAME_UNICODE,
-					  PR_EMAIL_ADDRESS_UNICODE,
+					  PR_SMTP_ADDRESS_UNICODE,
 					  PR_DISPLAY_TYPE,
 					  PR_OBJECT_TYPE,
 					  PR_ADDRTYPE_UNICODE,
@@ -768,7 +772,9 @@ exchange_mapi_util_get_gal (GPtrArray *contacts_array)
 		count += 0x2;
 		retval = GetGALTable(global_mapi_session, SPropTagArray, &SRowSet, count, ulFlags);
 		if ((!SRowSet) || (!(SRowSet->aRow))) {
-			return false;
+			UNLOCK ();
+			MAPIFreeBuffer (SPropTagArray);
+			return FALSE;
 		}
 		if (SRowSet->cRows) {
 			for (i = 0; i < SRowSet->cRows; i++) {
@@ -784,7 +790,9 @@ exchange_mapi_util_get_gal (GPtrArray *contacts_array)
 
 	MAPIFreeBuffer(SPropTagArray);
 
-	return true;
+	UNLOCK ();
+
+	return TRUE;
 	
 }
 
@@ -906,7 +914,9 @@ exchange_mapi_util_modify_recipients (TALLOC_CTX *mem_ctx, mapi_object_t *obj_me
 	}
 
 	/* Attempt to resolve names from the server */
+	LOCK ();
 	retval = ResolveNames (global_mapi_session, users, SPropTagArray, &SRowSet, &FlagList, 0);
+	UNLOCK ();
 	if (retval != MAPI_E_SUCCESS) {
 		mapi_errstr("ResolveNames", GetLastError());
 		goto cleanup;
@@ -1209,13 +1219,12 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 
 		for (i = 0; i < SRowSet.cRows; i++) {
 			mapi_object_t obj_message;
-			struct mapi_SPropValue_array properties_array;
+			struct mapi_SPropValue_array properties_array = {0};
 			const mapi_id_t *pfid;
 			const mapi_id_t	*pmid;
 			const bool *has_attach = NULL;
 			GSList *attach_list = NULL;
 			GSList *recip_list = NULL;
-			GSList *gal_list = NULL;
 			GSList *stream_list = NULL;
 			gboolean cb_retval = false;
 
@@ -1226,6 +1235,9 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 
 			has_attach = (const bool *) get_SPropValue_SRow_data(&SRowSet.aRow[i], PR_HASATTACH);
 
+			if (options & MAPI_OPTIONS_DONT_OPEN_MESSAGE)
+				goto relax;
+				
 			retval = OpenMessage(&obj_folder, *pfid, *pmid, &obj_message, 0);
 			if (retval != MAPI_E_SUCCESS) {
 				mapi_errstr("OpenMessage", GetLastError());
@@ -1261,19 +1273,21 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 
 			} else
 				retval = GetPropsAll (&obj_message, &properties_array);
-
+ relax:
 			if (retval == MAPI_E_SUCCESS) {
 				FetchItemsCallbackData *item_data;
 				uint32_t z;
 
-				/* just to get all the other streams */
-				for (z=0; z < properties_array.cValues; z++) {
-					if ((properties_array.lpProps[z].ulPropTag & 0xFFFF) == PT_BINARY && 
-					    (options & MAPI_OPTIONS_FETCH_GENERIC_STREAMS)) 
-					exchange_mapi_util_read_generic_stream (&obj_message, properties_array.lpProps[z].ulPropTag, &stream_list);
-				}
+				if ((options & MAPI_OPTIONS_DONT_OPEN_MESSAGE) == 0) {
+					/* just to get all the other streams */
+					for (z=0; z < properties_array.cValues; z++) {
+						if ((properties_array.lpProps[z].ulPropTag & 0xFFFF) == PT_BINARY && 
+						    (options & MAPI_OPTIONS_FETCH_GENERIC_STREAMS)) 
+						exchange_mapi_util_read_generic_stream (&obj_message, properties_array.lpProps[z].ulPropTag, &stream_list);
+					}
 
-				mapi_SPropValue_array_named(&obj_message, &properties_array);
+					mapi_SPropValue_array_named(&obj_message, &properties_array);
+				}
 
 				/* NOTE: stream_list, recipient_list and attach_list
 				   should be freed by the callback */
@@ -1296,7 +1310,8 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 				talloc_free (properties_array.lpProps);
 
 		loop_cleanup:
-			mapi_object_release(&obj_message);
+			if ((options & MAPI_OPTIONS_DONT_OPEN_MESSAGE) == 0)
+				mapi_object_release (&obj_message);
 
 			if (!cb_retval) break;
 		}
